import 'dart:async';
import 'dart:convert';

import 'package:uuid/uuid.dart';
import 'package:socket_io_client/socket_io_client.dart' as io;
import 'constants.dart' as constants;

// All socket connecting functions should go inside this class and
// expose a API to Comunia Client
class SocketClient {
  String url;
  String apiKey;
  String userId;
  io.Socket socket;
  Uuid uuidGenerator = Uuid();

  Stream onNewMessage;
  final StreamController _onNewMessageController = StreamController.broadcast();

  SocketClient(String url, String apiKey, String userId) {
    this.url = url;
    this.apiKey = apiKey;
    this.userId = userId;

    onNewMessage = _onNewMessageController.stream;
  }

  Future<dynamic> init () async {
    socket = io.io(url, <String, dynamic>{
      'transports': ['websocket'],
      'autoConnect': false
    });

    final _completer = Completer<dynamic>();
    socket.on('connect', (_) {
      var loginOptions = {
        'user_id': userId,
        'api_key': apiKey,
        'device_info': {
          'sdk_type': 'flutter'
        }
      };

      socket.emit(constants.EE_LOGIN, loginOptions);
    });
    socket.on(constants.EH_RECEIVE_MESSAGE, _onNewMessage);
    socket.on(constants.EH_LOGIN_RESPONSE, (data) async {
      print(data);
      if (data['response_code'] != 200) {
        return _completer.completeError(data['message']);
      }

      return _completer.complete(data);
    });

    socket.connect();
    return _completer.future;
  }

  void _sendMessage (options) async {
    socket.emit(constants.EE_SEND_MESSAGE, options);
    socket.on(constants.EH_SERVER_PING_BACK_FOR_MESSAGE, (data) async {
      _log(data);
      if (data['status'] != 200) {
        _log("$data['message_id'] failed with $data['description']");
      }

    });
  }

  Map<String, dynamic> _generateMessage (receiver, data, type, expireTime, messageId) {
    messageId ??= uuidGenerator.v4();
    var encodedData = base64.encode(utf8.encode(data));

    var options = {
      'message_id': messageId,
      'receiver': receiver,
      'data': encodedData,
      'type': type,
      'expire_time': expireTime
    };

    return options;
  }

  void sendNewMessage (toUser, message, {expireTime = constants.DEFAULT_EXPIRE_TYPE}) async {
    var options = _generateMessage(toUser, message, constants.MT_NEW_MESSAGE, expireTime, null);

    _sendMessage(options);
  }

  void sendOnStartTyping (toUser) async {
    var options = _generateMessage(toUser, 'Started typing...', constants.MT_START_TYPING, constants.DEFAULT_EXPIRE_TYPE, null);

    _sendMessage(options);
  }

  void sendOnStopTyping (toUser) async {
    var options = _generateMessage(toUser, 'Stopped typing...', constants.MT_STOP_TYPING, constants.DEFAULT_EXPIRE_TYPE, null);

    _sendMessage(options);
  }

  void sendMessageViewed (toUser, messageId) async {
    var options = _generateMessage(toUser, 'Seen', constants.MT_SEEN, constants.DEFAULT_EXPIRE_TYPE, messageId);

    _sendMessage(options);
  }

  void _onNewMessage (data) async {
    _log(data);
    _onNewMessageController.add(data);
  }
  void _log(data){
    print('comunia > $data');
  }
}